import datetime


class Field:
    def __init__(self, field=None, lookup=None):
        self._value = None
        self._field = field
        self._lookup = lookup

    def __set__(self, instance, value):
        self._value = value

    def __get__(self, instance, owner):
        if self._lookup:
            return {'__'.join((self._field, self._lookup)): self._value}
        else:
            return {self._field: self._value}


class CharField(Field):
    def __set__(self, instance, value):
        try:
            self._value = str(value)
        except Exception as err:
            raise ValueError('查询参数转化为字符串失败!')


class IntField(Field):
    def __set__(self, instance, value):
        try:
            self._value = int(value)
        except Exception as err:
            raise ValueError('值: {} 转化为整型失败!'.format(value))


class ArrayField(Field):
    def __set__(self, instance, value):
        if isinstance(value, list):
            self._value = value
        elif isinstance(value, str):
            self._value = value.split(',')
        elif value is None:
            self._value = []
        else:
            raise ValueError('值: {} 要求是一个数组，或者逗号分割的字符串'.format(str(value)))


class BoolField(Field):
    def __set__(self, instance, value):
        false_li, true_li = [False, 'false', '0', 0], [True, 'true', '1', 1]
        value = str(value).lower() if isinstance(value, str) else value
        if value not in false_li + true_li:
            raise ValueError('值: {} 必须为 true 或者 false。'.format(str(value)))
        self._value = False if value in false_li else True


class DateTimeField(Field):
    """本函数支持了使用不完整的日期时间来查询DateTimeField"""

    def __set__(self, instance, value):
        try:
            value = value.strip()
            date, *time = value.split(' ')
            # 如果仅仅提供了日期
            if not time:
                date_time = datetime.datetime.strptime(value, '%Y-%m-%d')
                timedelta = datetime.timedelta(days=1)
                self._value = [date_time, date_time + timedelta]
                self._lookup = 'range'
            # 如果提供了时期时间，时间类型有三种1.只提供了小时 2.只提供了分钟3.完整的时分秒
            else:
                time = time[0]
                time_type = len(time.split(':'))
                if time_type == 1:
                    date_time = datetime.datetime.strptime(value, '%Y-%m-%d %H')
                    timedelta = datetime.timedelta(hours=1)
                    self._value = [date_time, date_time + timedelta]
                    self._lookup = 'range'
                elif time_type == 2:
                    date_time = datetime.datetime.strptime(value, '%Y-%m-%d %H:%M')
                    timedelta = datetime.timedelta(minutes=1)
                    self._value = [date_time, date_time + timedelta]
                    self._lookup = 'range'
                elif time_type == 3:
                    date_time = datetime.datetime.strptime(value, '%Y-%m-%d %H:%M:%S')
                    self._value = date_time
                else:
                    raise ValueError
        except Exception as err:
            raise ValueError('{} 必须为合法的日期时间格式，请使用 YYYY-MM-DD 或 '
                             'YYYY-MM-DD HH:MM:SS'.format(value))



from .fields import Field, ManyToManyField, CrossTable
from django.db.models import QuerySet, Model
from collections import Iterable


class SerializerMeta(type):
    def __new__(mcs, name, bases, act):
        parents = [b for b in bases if isinstance(b, SerializerMeta)]
        if not parents:
            return super().__new__(mcs, name, bases, act)

        # 当filed被设置为None时，使用属性名作为filed的值
        for attr_name, attr in act.items():
            if isinstance(attr, Field) and not attr.field:
                attr.field = attr_name

        # 记录多对多字段和跨表
        many_or_cross = []
        for attr_name, attr in act.items():
            if isinstance(attr, ManyToManyField) or isinstance(attr, CrossTable):
                many_or_cross.append(attr_name)
        # 操作和添加Meta
        attr_meta = act.pop('Meta', None)  # pop掉Meta是为了拿到父类的Meta信息
        new_class = super().__new__(mcs, name, bases, act)
        meta = attr_meta or getattr(new_class, 'Meta', None)
        only, defer = getattr(meta, 'only', []), getattr(meta, 'defer', [])
        meta_many_cross = getattr(meta, '_many_or_cross', [])
        meta_attr = {'only': only, 'defer': defer, '_many_or_cross': many_or_cross + meta_many_cross}
        new_class.Meta = type('Meta', (), meta_attr)  # 给每个类动态生成一个新的Meta类
        return new_class

    def __call__(cls, set_or_obj):
        self = super().__call__()
        results = self._serialize(set_or_obj)
        return results


class Serializer(metaclass=SerializerMeta):

    def _serialize(self, set_or_obj):
        """序列化queryset或model_obj"""
        if isinstance(set_or_obj, Iterable):
            return [self._to_dict(obj) for obj in set_or_obj]

        elif isinstance(set_or_obj, Model):
            return self._to_dict(set_or_obj)

        else:
            raise TypeError('{}类型不支持序列化'.format(type(set_or_obj)))

    # model_obj处理成字典
    def _to_dict(self, model_obj):
        obj_dict = {}

        for field in model_obj._meta.fields:
            # only被定义时，忽略only以外的全部字段
            if self.Meta.only and field.name not in self.Meta.only:
                continue
            # 只在没定义only时defer生效，only一旦定义defer失效
            if not self.Meta.only and field.name in self.Meta.defer:
                continue
            field_value = getattr(model_obj, field.name)  # 获取字段的值
            handle = getattr(self, field.name, Field.default)  # 获取（在描述符中）字段的处理方法
            if field.choices:  # 获取该字段的display
                obj_dict[field.name + '_display'] = dict(field.choices).get(field_value, None)
            obj_dict[field.name] = handle(field_value)
        # 处理多对多字段以及跨表
        for many_or_table_name in self.Meta._many_or_cross:
            handle = getattr(self, many_or_table_name)
            obj_dict[many_or_table_name] = handle(model_obj)

        return obj_dict

    class Meta:
        """如果外键没有定义序列化类，那么only和defer对外键同样生效,如序列化多对多字段一定要设置序列化类"""
        _many_or_cross = []
        only = []  # 当定义了only时defer失效
        # 本次序列化排除的字段
        defer = []



from .fields import Field
from django.db.models import QuerySet


class FilterMeta(type):
    def __new__(mcs, name, bases, act):
        # 命名规范和设置field默认值
        for attr_name, attr in act.items():
            if isinstance(attr, Field):

                if attr_name.startswith('_') or attr_name[0].isupper():
                    raise TypeError('{}中的{},不能为下划线开头，不能大写字母开头'.format(name, attr_name))

                # 当filed被设置为None时，使用属性名作为filed的值
                if not attr._field:
                    attr._field = attr_name
        return super().__new__(mcs, name, bases, act)

    def __init__(cls, name, bases, act):
        super().__init__(name, bases, act)
        # 约束类的定义
        cls.const(name, bases, act)

    def __call__(cls, *args, **kwargs):
        self = super().__call__(*args, **kwargs)
        return self._queryset

    def const(cls, name, bases, act):
        if '_base' in act and act['_base']:
            return
        # Filter中的class Meta 必须定义model
        meta = getattr(cls, 'Meta', None)
        if not getattr(meta, 'model', None):
            raise TypeError('{}的Meta.model不能为None'.format(name))


class Filter(metaclass=FilterMeta):
    _base = True  # _base=True意味着该类只作被继承使用，不会在业务中直接使用

    def __init__(self, request, queryset=None):
        query = request.POST or request.GET
        # queryset为None时取Meta.model.objects
        # 这里不能用or运算，or运算会将空queryset认为None
        if queryset is None:
            self._queryset = self.Meta.model.objects
        elif isinstance(queryset, QuerySet):
            self._queryset = queryset
        else:
            raise ValueError('{} queryset必须是QuerySet Object'.format(self.__class__.__name__))
        self._query = {}
        for key, value in query.items():
            if not isinstance(getattr(self, key, None), dict):
                continue
            setattr(self, key, value)
            self._query.update(getattr(self, key))

        self._queryset = self._queryset.filter(**self._query)

    class Meta:
        model = None



import json
from json.decoder import JSONDecodeError


class Field:
    def __init__(self, field=None):
        self.field = field

    def __get__(self, instance, owner):
        return self.call_back

    def call_back(self, value):
        # 整型和None类型不转换，布尔类型也是整型
        if isinstance(value, int) or value is None:
            return value
        else:
            return str(value)

    # 当字段未定义描述符时调用该默认函数
    @classmethod
    def default(cls, value):
        # 整型和None类型不转换，布尔类型也是整型
        if isinstance(value, int) or value is None:
            return value
        else:
            return str(value)


class Foreignkey(Field):
    def __init__(self, serialize_class=None, field=None):
        self.owner = None
        self.serialize_class = serialize_class
        super().__init__(field)

    def __get__(self, instance, owner):
        self.owner = owner
        return self.call_back

    def call_back(self, value):
        if not value:
            return None
        if self.serialize_class:
            serialize_class = self.serialize_class
        else:
            serialize_class = self.owner
        return serialize_class(value)


class JsonTextField(Field):

    def call_back(self, value):
        if not value:
            return None
        try:
            return json.loads(value)
        except JSONDecodeError:
            TypeError('json loads发生错误,字段:{} ！'.format(self.field))


class ImgField(Field):
    def __init__(self, host, media_url, field=None):
        self.host, self.media_url = host, media_url
        super().__init__(field=field)

    def call_back(self, value):
        if not value:
            return None
        return self.host + self.media_url + str(value)


class DateTimeField(Field):

    def call_back(self, value):
        if not value:
            return value
        return str(value)[0:19]


class ManyToManyField(Field):
    def __init__(self, serialize_class, field=None):
        self.instance = None
        self.serialize_class = serialize_class
        super().__init__(field)

    def __get__(self, instance, owner):
        self.instance = instance
        return self.call_back

    def call_back(self, model_obj):
        hook_name = '_'.join(('hook', self.field))
        hook_func = getattr(self.instance, hook_name, None)
        if hook_func:
            query_set = hook_func(model_obj)
        else:
            many_obj = getattr(model_obj, self.field, None)
            query_set = many_obj.all() if many_obj else ()
        return self.serialize_class(query_set)


class CrossTable(ManyToManyField):
    def call_back(self, model_obj):
        hook_name = '_'.join(('hook', self.field))
        hook_func = getattr(self.instance, hook_name, None)
        if hook_func:
            query_set = hook_func(model_obj)
        else:
            cross_obj = getattr(model_obj, '_'.join((self.field, 'set')), None)
            query_set = cross_obj.all() if cross_obj else ()
        return self.serialize_class(query_set)
